/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

/**
 * At this moment the rf switch port is implemented which ioperm(); 
 **/

#define _GNU_SOURCE
/* #define SW_GOOD_HARDWARE */

#include <stdio.h>
#include <stdint.h>
#include <sys/io.h>
#include "my_stdio.h"


#ifdef SW_GOOD_HARDWARE

# define SWITCH_1  0x01
# define SWITCH_2  0x02
# define SWITCH_3  0x04
# define CAL_DIODE 0x08

# define ANTENNA_1     (0)
# define ANTENNA_1_CAL (ANTENNA_1|CAL_DIODE)
# define ANTENNA_2     (SWITCH_1)
# define ANTENNA_2_CAL (ANTENNA_2|CAL_DIODE)
# define ANTENNA_3     (SWITCH_2)
# define ANTENNA_3_CAL (ANTENNA_3|CAL_DIODE)
# define ANTENNA_4     (SWITCH_3)
# define ANTENNA_4_CAL (ANTENNA_4|CAL_DIODE)

# define MK_V(v) ((unsigned char) (v))

#else

# define SWITCH_1  0x80
# define SWITCH_2  0x40
# define SWITCH_3  0x20
# define CAL_DIODE 0x01

# define SWITCH_MASK (SWITCH_1|SWITCH_2|SWITCH_3/*|CAL_DIODE*/)

# define ANTENNA_1     (SWITCH_2|SWITCH_3)
# define ANTENNA_1_CAL (ANTENNA_1|CAL_DIODE)
# define ANTENNA_2     (SWITCH_1|SWITCH_3)
# define ANTENNA_2_CAL (ANTENNA_2|CAL_DIODE)
# define ANTENNA_3     (0)
# define ANTENNA_3_CAL (ANTENNA_3|CAL_DIODE)
# define ANTENNA_4     (0) /* FIXME: (SWITCH_3) */
# define ANTENNA_4_CAL (ANTENNA_4|CAL_DIODE)

# define MK_V(v) ((unsigned char) (SWITCH_MASK ^ v))

#endif /* SW_GOOD_HARDWARE */

static int io_perm = 0;
static int as_port;


static void my_outb(unsigned char value, unsigned short int port)
{
	PVERB(PINFO,"# outb 0x%02x, 0x%03x\n", (unsigned char) value, port);
#ifndef DEMO_MODE
	outb(value, port);
#endif
	return;
}


int antenna_switchs_open(int port)
{
	as_port = port;
#ifdef DEMO_MODE
	io_perm = 1;
	return 0;
#endif
	/* Get access to the ports */
	if (as_port < 0x3ff ) {	
		if (ioperm(as_port, 1, 1)) {
			perror("ioperm");
			return -1;
		}
	}else{
		if (iopl(3) != 0) {
			perror("iopl");
			return -1;
		}
	}
	io_perm = 1;

	return 0;
}


int antenna_switchs_close(void)
{
	/* switch off rf sw */
	if (io_perm) 
		my_outb(MK_V(0), as_port);
#ifndef DEMO_MODE
	/* Release access to the ports */
	if (ioperm(as_port, 1, 0)) {
		perror("ioperm"); 
		return -1;
	}
#endif
	io_perm = 0;

	return 0;
}


int antenna_switchs_set(char switchs)
{
	
	if (!io_perm) return -1;

	switch (switchs) {
	case 'A': my_outb(MK_V(ANTENNA_1),     as_port); break;
	case 'B': my_outb(MK_V(ANTENNA_1_CAL), as_port); break;
	case 'C': my_outb(MK_V(ANTENNA_2),     as_port); break;
	case 'D': my_outb(MK_V(ANTENNA_2_CAL), as_port); break;
	case 'E': my_outb(MK_V(ANTENNA_3),     as_port); break;
	case 'F': my_outb(MK_V(ANTENNA_3_CAL), as_port); break;
	case 'G': my_outb(MK_V(ANTENNA_4),     as_port); break;
	case 'H': my_outb(MK_V(ANTENNA_4_CAL), as_port); break;
	default:
		return -1;
	}

	return 0;
}


/* Only for testing */
int antenna_byte_set(char b)
{
	if (!io_perm) return -1;
	
	PVERB(PINFO," out 0x%02x\n", (unsigned char) b);
	my_outb(b, as_port);

	return 0;
}
